//
// LPTK model password.rs
// Copyright (C) 2025 Óscar García Amor <ogarcia@connectical.com>
// Distributed under terms of the GNU GPLv3 license.
//

use std::cell::RefCell;

use gtk::prelude::*;
use adw::subclass::prelude::*;
use gtk::glib;

mod imp {
    use super::*;

    #[derive(Debug, Default, glib::Properties)]
    #[properties(wrapper_type = super::Password)]
    pub struct Password {
        #[property(name = "id", get, set, type = String, member = id)]
        #[property(name = "site", get, set, type = String, member = site)]
        #[property(name = "login", get, set, type = String, member = login)]
        #[property(name = "lower", get, set, type = bool, member = lower)]
        #[property(name = "upper", get, set, type = bool, member = upper)]
        #[property(name = "symbols", get, set, type = bool, member = symbols)]
        #[property(name = "digits", get, set, type = bool, member = digits)]
        #[property(name = "length", get, set, type = u8, member = length)]
        #[property(name = "counter", get, set, type = u32, member = counter)]
        pub data: RefCell<PasswordData>,
    }

    #[glib::object_subclass]
    impl ObjectSubclass for Password {
        const NAME: &'static str = "Password";
        type Type = super::Password;
    }

    #[glib::derived_properties]
    impl ObjectImpl for Password {}
}

glib::wrapper! {
    pub struct Password(ObjectSubclass<imp::Password>);
}

impl Password {
    pub fn new(id: String, site: String, login: String, lower: bool, upper: bool, digits: bool, symbols: bool, length: u8, counter: u32) -> Self {
        glib::Object::builder()
            .property("id", id)
            .property("site", site)
            .property("login", login)
            .property("lower", lower)
            .property("upper", upper)
            .property("digits", digits)
            .property("symbols", symbols)
            .property("length", length)
            .property("counter", counter)
            .build()
    }
}

impl Default for Password {
    fn default() -> Self {
        Self::new(String::new(), String::new(), String::new(), false, false, false, false, 0, 0)
    }
}

#[derive(Debug, Default)]
pub struct PasswordData {
    pub id: String,
    pub site: String,
    pub login: String,
    pub lower: bool,
    pub upper: bool,
    pub symbols: bool,
    pub digits: bool,
    pub length: u8,
    pub counter: u32
}
