//
// LPTK ui password.rs
// Copyright (C) 2025 Óscar García Amor <ogarcia@connectical.com>
// Distributed under terms of the GNU GPLv3 license.
//

use std::cell::RefCell;

use gtk::prelude::*;
use adw::subclass::prelude::*;
use gtk::glib;

use crate::model::Password;

mod imp {
    use super::*;

    #[derive(Debug, Default, gtk::CompositeTemplate)]
    #[template(resource = "/me/ogarcia/lptk/gtk/password.ui")]
    pub struct FragmentPassword {
        // Template widgets
        #[template_child]
        pub site: TemplateChild<gtk::Label>,
        #[template_child]
        pub login: TemplateChild<gtk::Label>,
        #[template_child]
        pub lower: TemplateChild<gtk::Frame>,
        #[template_child]
        pub upper: TemplateChild<gtk::Frame>,
        #[template_child]
        pub digits: TemplateChild<gtk::Frame>,
        #[template_child]
        pub symbols: TemplateChild<gtk::Frame>,
        #[template_child]
        pub length: TemplateChild<gtk::Label>,
        #[template_child]
        pub counter: TemplateChild<gtk::Label>,
        // Vector holding the bindings to properties of Password
        pub bindings: RefCell<Vec<glib::Binding>>,
    }

    #[glib::object_subclass]
    impl ObjectSubclass for FragmentPassword {
        const NAME: &'static str = "FragmentPassword";
        type Type = super::FragmentPassword;
        type ParentType = gtk::Box;

        fn class_init(klass: &mut Self::Class) {
            klass.bind_template();
        }

        fn instance_init(obj: &glib::subclass::InitializingObject<Self>) {
            obj.init_template();
        }
    }

    impl ObjectImpl for FragmentPassword {}
    impl WidgetImpl for FragmentPassword {}
    impl BoxImpl for FragmentPassword {}
}

glib::wrapper! {
    pub struct FragmentPassword(ObjectSubclass<imp::FragmentPassword>)
        @extends gtk::Box, gtk::Widget,
        @implements gtk::Accessible, gtk::Buildable, gtk::ConstraintTarget, gtk::Orientable;
}

impl FragmentPassword {
    pub fn new() -> Self {
        glib::Object::builder().build()
    }

     pub fn bind(&self, password: &Password) {
        let site = self.imp().site.get();
        let login = self.imp().login.get();
        let upper = self.imp().upper.get();
        let lower = self.imp().lower.get();
        let digits = self.imp().digits.get();
        let symbols = self.imp().symbols.get();
        let length = self.imp().length.get();
        let counter = self.imp().counter.get();
        let mut bindings = self.imp().bindings.borrow_mut();

        // Bind password.site to fragment_password.site.label
        let site_binding = password
            .bind_property("site", &site, "label")
            .sync_create()
            .build();
        // Save binding
        bindings.push(site_binding);

        // Bind password.login to fragment_password.login.label
        let login_binding = password
            .bind_property("login", &login, "label")
            .sync_create()
            .build();
        // Save binding
        bindings.push(login_binding);

        // Bind password.upper to fragment_password.upper.visible
        let upper_binding = password
            .bind_property("upper", &upper, "visible")
            .sync_create()
            .build();
        // Save binding
        bindings.push(upper_binding);

        // Bind password.lower to fragment_password.lower.visible
        let lower_binding = password
            .bind_property("lower", &lower, "visible")
            .sync_create()
            .build();
        // Save binding
        bindings.push(lower_binding);

        // Bind password.digits to fragment_password.digits.visible
        let digits_binding = password
            .bind_property("digits", &digits, "visible")
            .sync_create()
            .build();
        // Save binding
        bindings.push(digits_binding);

        // Bind password.symbols to fragment_password.symbols.visible
        let symbols_binding = password
            .bind_property("symbols", &symbols, "visible")
            .sync_create()
            .build();
        // Save binding
        bindings.push(symbols_binding);

        // Bind password.length to fragment_password.length.label
        let length_binding = password
            .bind_property("length", &length, "label")
            .sync_create()
            .build();
        // Save binding
        bindings.push(length_binding);

        // Bind password.counter to fragment_password.counter.label
        let counter_binding = password
            .bind_property("counter", &counter, "label")
            .sync_create()
            .build();
        // Save binding
        bindings.push(counter_binding);
     }

     pub fn unbind(&self) {
        for binding in self.imp().bindings.borrow_mut().drain(..) {
            binding.unbind();
        }
    }
}
